from .assets_form_validator import HealthEconomicsAssetsFormValidator
from .household_head_form_validator import HealthEconomicsHouseholdHeadFormValidator
from .income_form_validator import HealthEconomicsIncomeFormValidator
from .patient_form_validator import HealthEconomicsPatientFormValidator
from .property_form_validator import HealthEconomicsPropertyFormValidator
from .simple import SimpleFormValidatorMixin



from __future__ import annotations

from django import forms
from django.test import TestCase
from edc_constants.constants import COMPLETE, FEMALE, NO, NOT_APPLICABLE, OTHER, YES
from edc_metadata import NOT_REQUIRED, REQUIRED, site_metadata_rules
from edc_metadata.tests import CrfTestHelper
from edc_utils import get_utcnow
from edc_utils.test_case_mixins.longitudinal_test_case_mixin import (
    LongitudinalTestCaseMixin,
)

from edc_he.constants import WIFE_HUSBAND
from edc_he.form_validators import HealthEconomicsHouseholdHeadFormValidator
from edc_he.models import (
    Education,
    EmploymentType,
    Ethnicities,
    HealthEconomicsHouseholdHead,
    InsuranceTypes,
    Religions,
)
from edc_he.rule_groups import HealthEconomicsRuleGroup as BaseHealthEconomicsRuleGroup
from edc_he.rule_groups import Predicates

from ...utils import get_patient_model
from ..visit_schedule import visit_schedule


def get_m2m_qs(model_cls, name: str = None):
    if name:
        qs = model_cls.objects.filter(name=name)
    else:
        name = model_cls.objects.all()[0].name
        qs = model_cls.objects.filter(name=name).exclude(name=NOT_APPLICABLE)
    return qs


def get_obj(model_cls, name: str = None):
    if name:
        obj = model_cls.objects.get(name=name)
    else:
        obj = model_cls.objects.all()[0]
    return obj


class HealthEconomicsHouseholdHeadTests(LongitudinalTestCaseMixin, CrfTestHelper, TestCase):
    visit_schedule = visit_schedule

    def setUp(self) -> None:
        self.subject_identifier = self.enroll()
        self.create_visits(self.subject_identifier)

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        class HealthEconomicsRuleGroup(BaseHealthEconomicsRuleGroup):
            class Meta:
                app_label = "edc_he"
                source_model = "edc_visit_tracking.subjectvisit"
                predicates = Predicates()

        site_metadata_rules.register(HealthEconomicsRuleGroup)

    def get_cleaned_data(self, **kwargs) -> dict:
        cleaned_data = dict(
            subject_visit=self.subject_visit_baseline,
            report_datetime=get_utcnow(),
            hh_count=5,
            hh_minors_count=1,
            hoh=NO,
            relationship_to_hoh=WIFE_HUSBAND,
            relationship_to_hoh_other=None,
            hoh_gender=FEMALE,
            hoh_age=25,
            hoh_religion=get_obj(Religions),
            hoh_religion_other=None,
            hoh_ethnicity=get_obj(Ethnicities),
            hoh_ethnicity_other=None,
            hoh_education=get_obj(Education),
            hoh_education_other=None,
            hoh_employment_status="",
            hoh_employment_type=get_obj(EmploymentType),
            hoh_employment_type_other=None,
            hoh_marital_status=OTHER,
            hoh_marital_status_other="blah",
            hoh_insurance=get_m2m_qs(InsuranceTypes),
            hoh_insurance_other=None,
            crf_status=COMPLETE,
            crf_status_comments="",
        )
        cleaned_data.update(**kwargs)
        return cleaned_data

    def test_cleaned_data_ok(self):
        instance = HealthEconomicsHouseholdHead()
        cleaned_data = self.get_cleaned_data()
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data,
            instance=instance,
            model=HealthEconomicsHouseholdHead,
        )
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

    def test_relationship_to_hoh_applicable_if_not_hoh(self):
        instance = HealthEconomicsHouseholdHead()
        opts = dict(
            instance=instance,
            model=HealthEconomicsHouseholdHead,
        )

        not_applicable_opts = dict(
            hoh_religion=get_obj(Religions, NOT_APPLICABLE),
            hoh_ethnicity=get_obj(Ethnicities, NOT_APPLICABLE),
            hoh_education=get_obj(Education, NOT_APPLICABLE),
            hoh_employment_type=get_obj(EmploymentType, NOT_APPLICABLE),
            hoh_insurance=get_m2m_qs(InsuranceTypes, NOT_APPLICABLE),
        )
        cleaned_data = self.get_cleaned_data(
            hoh=YES,
            relationship_to_hoh=None,
            **not_applicable_opts,
        )
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        with self.assertRaises(forms.ValidationError):
            form_validator.validate()
        self.assertIn("relationship_to_hoh", form_validator._errors)
        self.assertIn(
            "This field is not applicable",
            str(form_validator._errors.get("relationship_to_hoh")),
        )

        cleaned_data = self.get_cleaned_data(
            hoh=YES,
            relationship_to_hoh=NOT_APPLICABLE,
            **not_applicable_opts,
        )
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

        cleaned_data.update(hoh=NO, relationship_to_hoh=None)
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        with self.assertRaises(forms.ValidationError):
            form_validator.validate()
        self.assertIn("relationship_to_hoh", form_validator._errors)
        self.assertIn(
            "This field is applicable", str(form_validator._errors.get("relationship_to_hoh"))
        )

        cleaned_data.update(hoh=NO, relationship_to_hoh=NOT_APPLICABLE)
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        with self.assertRaises(forms.ValidationError):
            form_validator.validate()
        self.assertIn("relationship_to_hoh", form_validator._errors)
        self.assertIn(
            "This field is applicable", str(form_validator._errors.get("relationship_to_hoh"))
        )

        applicable_opts = dict(
            hoh_religion=get_obj(Religions),
            hoh_ethnicity=get_obj(Ethnicities),
            hoh_education=get_obj(Education),
            hoh_employment_type=get_obj(EmploymentType),
            hoh_insurance=get_m2m_qs(InsuranceTypes),
        )

        cleaned_data.update(hoh=NO, relationship_to_hoh=WIFE_HUSBAND, **applicable_opts)
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

        cleaned_data.update(hoh=NO, relationship_to_hoh=OTHER, **applicable_opts)
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        with self.assertRaises(forms.ValidationError):
            form_validator.validate()
        self.assertIn("relationship_to_hoh_other", form_validator._errors)

        cleaned_data.update(
            hoh=NO,
            relationship_to_hoh=OTHER,
            relationship_to_hoh_other="blah",
            **applicable_opts,
        )
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

    def test_hoh_religion_other(self):
        instance = HealthEconomicsHouseholdHead()
        opts = dict(
            instance=instance,
            model=HealthEconomicsHouseholdHead,
        )
        applicable_opts = dict(
            hoh_ethnicity=get_obj(Ethnicities),
            hoh_education=get_obj(Education),
            hoh_employment_type=get_obj(EmploymentType),
            hoh_insurance=get_m2m_qs(InsuranceTypes),
        )
        cleaned_data = self.get_cleaned_data()
        cleaned_data.update(
            hoh=NO,
            relationship_to_hoh=WIFE_HUSBAND,
            hoh_religion=None,
            **applicable_opts,
        )
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        with self.assertRaises(forms.ValidationError):
            form_validator.validate()
        self.assertIn("hoh_religion", form_validator._errors)
        self.assertIn(
            "This field is applicable",
            str(form_validator._errors.get("hoh_religion")),
        )

        cleaned_data = self.get_cleaned_data(
            hoh=NO,
            relationship_to_hoh=WIFE_HUSBAND,
            hoh_religion=get_obj(Religions, OTHER),
            **applicable_opts,
        )
        form_validator = HealthEconomicsHouseholdHeadFormValidator(
            cleaned_data=cleaned_data, **opts
        )
        with self.assertRaises(forms.ValidationError):
            form_validator.validate()
        self.assertIn("hoh_religion_other", form_validator._errors)
        self.assertIn(
            "This field is required",
            str(form_validator._errors.get("hoh_religion_other")),
        )

    def test_patient_required_if_patient_is_not_hoh(self):
        cleaned_data = self.get_cleaned_data(
            hoh=NO,
            relationship_to_hoh=WIFE_HUSBAND,
        )
        del cleaned_data["hoh_insurance"]
        hoh_obj = HealthEconomicsHouseholdHead.objects.create(**cleaned_data)
        hoh_obj.hoh_insurance.add(get_obj(InsuranceTypes))

        qs = self.crf_metadata_obj(get_patient_model(), REQUIRED, "1000")
        self.assertTrue(qs.exists())

    def test_patient_required_if_patient_is_hoh(self):
        cleaned_data = self.get_cleaned_data(
            hoh=YES,
            relationship_to_hoh=NOT_APPLICABLE,
        )
        del cleaned_data["hoh_insurance"]
        hoh_obj = HealthEconomicsHouseholdHead.objects.create(**cleaned_data)
        hoh_obj.hoh_insurance.add(get_obj(InsuranceTypes))

        qs = self.crf_metadata_obj(get_patient_model(), NOT_REQUIRED, "1000")
        self.assertTrue(qs.exists())



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from edc_constants.constants import YES

from ..utils import (
    get_assets_model_cls,
    get_household_head_model_cls,
    get_income_model_cls,
    get_patient_model_cls,
    get_property_model_cls,
)


class HealthEconomicsModelFormMixin:
    def clean(self):
        self.raise_if_he_household_head_required()
        self.raise_if_he_patient_required()
        self.raise_if_he_assets_required()
        self.raise_if_he_property_required()
        return super().clean()

    @property
    def household_head(self):
        return get_household_head_model_cls().objects.get(
            subject_visit__subject_identifier=self.subject_identifier
        )

    @property
    def is_hoh(self):
        return self.household_head.hoh == YES

    def raise_if_he_household_head_required(self):
        if self._meta.model != get_household_head_model_cls():
            try:
                self.household_head
            except ObjectDoesNotExist:
                raise forms.ValidationError(
                    f"Complete {get_household_head_model_cls()._meta.verbose_name} CRF first."
                )

    def raise_if_he_patient_required(self):
        if not self.is_hoh and self._meta.model in [
            get_assets_model_cls(),
            get_income_model_cls(),
            get_property_model_cls(),
        ]:
            try:
                get_patient_model_cls().objects.get(
                    subject_visit__subject_identifier=self.subject_identifier
                )
            except ObjectDoesNotExist:
                raise forms.ValidationError(
                    f"Complete {get_patient_model_cls()._meta.verbose_name} CRF first."
                )

    def raise_if_he_assets_required(self):
        if self._meta.model in [
            get_income_model_cls(),
            get_property_model_cls(),
        ]:
            try:
                get_assets_model_cls().objects.get(
                    subject_visit__subject_identifier=self.subject_identifier
                )
            except ObjectDoesNotExist:
                raise forms.ValidationError(
                    f"Complete {get_assets_model_cls()._meta.verbose_name} CRF first."
                )

    def raise_if_he_property_required(self):
        if self._meta.model in [
            get_income_model_cls(),
        ]:
            try:
                get_property_model_cls().objects.get(
                    subject_visit__subject_identifier=self.subject_identifier
                )
            except ObjectDoesNotExist:
                raise forms.ValidationError(
                    f"Complete {get_property_model_cls()._meta.verbose_name} CRF first."
                )



from dateutil.relativedelta import relativedelta
from edc_visit_schedule.schedule import Schedule
from edc_visit_schedule.visit import Crf, FormsCollection, Visit
from edc_visit_schedule.visit_schedule import VisitSchedule

from edc_he.utils import (
    get_assets_model,
    get_household_head_model,
    get_income_model,
    get_patient_model,
    get_property_model,
)

crfs = FormsCollection(
    Crf(show_order=10, model=get_household_head_model(), required=True),
    Crf(show_order=20, model=get_patient_model(), required=True),
    Crf(show_order=30, model=get_assets_model(), required=True),
    Crf(show_order=40, model=get_income_model(), required=True),
    Crf(show_order=50, model=get_property_model(), required=True),
)

visit0 = Visit(
    code="1000",
    title="Day 1",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    crfs=crfs,
)

visit1 = Visit(
    code="2000",
    title="Day 2",
    timepoint=1,
    rbase=relativedelta(days=1),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    crfs=crfs,
)

visit2 = Visit(
    code="3000",
    title="Day 3",
    timepoint=2,
    rbase=relativedelta(days=2),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    crfs=crfs,
)

visit3 = Visit(
    code="4000",
    title="Day 4",
    timepoint=3,
    rbase=relativedelta(days=3),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    crfs=crfs,
)

schedule = Schedule(
    name="schedule",
    onschedule_model="visit_schedule_app.onschedule",
    offschedule_model="visit_schedule_app.offschedule",
    appointment_model="edc_appointment.appointment",
    consent_model="visit_schedule_app.subjectconsent",
)

schedule.add_visit(visit0)
schedule.add_visit(visit1)
schedule.add_visit(visit2)
schedule.add_visit(visit3)

visit_schedule = VisitSchedule(
    name="visit_schedule",
    offstudy_model="visit_schedule_app.subjectoffstudy",
    death_report_model="visit_schedule_app.deathreport",
)

visit_schedule.add_schedule(schedule)



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import YES_NO_DONT_KNOW, YES_NO_DONT_KNOW_DWTA

from ..choices import FINANCIAL_STATUS, REMITTANCE_CURRENCY_CHOICES, STATUS
from .factory import income_model_mixin_factory


class IncomeModelMixin(income_model_mixin_factory(), models.Model):
    external_remit_currency = models.CharField(
        verbose_name="In what currency do you receive remittances?",
        max_length=50,
        choices=REMITTANCE_CURRENCY_CHOICES,
        null=True,
        blank=True,
    )

    external_remit_currency_other = models.CharField(
        verbose_name="If OTHER currency, specify ...",
        max_length=50,
        null=True,
        blank=True,
    )

    more_sources_other = models.CharField(
        verbose_name="If YES additional sources, specify ...",
        max_length=50,
        null=True,
        blank=True,
    )

    external_dependents = models.IntegerField(
        verbose_name=(
            "Outside of this household, how many other people depend on this "
            "household’s income?"
        ),
        validators=[MinValueValidator(0), MaxValueValidator(15)],
        help_text="Insert '0' if no dependents other than the members in the household roster",
    )

    income_enough = models.CharField(
        verbose_name=(
            "Thinking about the income for this household, do you "
            "believe that it is enough money to cover your daily living "
            "needs and obligations?"
        ),
        max_length=15,
        choices=YES_NO_DONT_KNOW,
    )

    financial_status = models.CharField(
        verbose_name="Would you say your household's financial situation is?",
        max_length=25,
        choices=STATUS,
    )

    financial_status_compare = models.CharField(
        verbose_name=(
            "How would you rate your household’s financial situation compared with others in "
            "your community?"
        ),
        max_length=25,
        choices=FINANCIAL_STATUS,
    )

    household_debt = models.CharField(
        verbose_name=(
            "Does your household or any members of the household have current debt "
            "or outstanding loans?"
        ),
        max_length=25,
        choices=YES_NO_DONT_KNOW_DWTA,
    )

    household_debt_value = models.IntegerField(
        verbose_name="What is the approximate total amount of this debt or loan(s)?",
        validators=[MinValueValidator(1), MaxValueValidator(9999999999)],
        null=True,
        blank=True,
        help_text="In local currency",
    )

    class Meta:
        abstract = True



# Generated by Django 4.2.3 on 2023-08-07 22:47

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_he", "0009_healtheconomicsassets_solar_panels_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="healtheconomicsproperty",
            name="land_surface_area",
            field=models.IntegerField(
                blank=True,
                help_text="Surface area in acres or m2. (Note 1 hector ~= 2.5 acres)",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999999999),
                ],
                verbose_name="Surface area",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicsproperty",
            name="land_surface_area_units",
            field=models.CharField(
                blank=True,
                choices=[
                    ("hectares", "hectares"),
                    ("acres", "acres"),
                    ("sq_meters", "sq. meters"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                null=True,
                verbose_name="Surface area (units)",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsproperty",
            name="land_surface_area",
            field=models.IntegerField(
                blank=True,
                help_text="Surface area in acres or m2. (Note 1 hector ~= 2.5 acres)",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999999999),
                ],
                verbose_name="Surface area",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsproperty",
            name="land_surface_area_units",
            field=models.CharField(
                blank=True,
                choices=[
                    ("hectares", "hectares"),
                    ("acres", "acres"),
                    ("sq_meters", "sq. meters"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=15,
                null=True,
                verbose_name="Surface area (units)",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_additional_known",
            field=models.CharField(
                default="QUESTION_RETIRED",
                help_text="Use cash equivalent in local currency",
                max_length=25,
                verbose_name="Do you know about how much is this worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_owner",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Do not know"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="Do you own any land.",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999999999),
                ],
                verbose_name="About how much this is worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsproperty",
            name="land_value_known",
            field=models.CharField(
                default="QUESTION_RETIRED",
                help_text="Use cash equivalent in local currency",
                max_length=25,
                verbose_name="Do you know about how much this is worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsproperty",
            name="land_additional_known",
            field=models.CharField(
                default="QUESTION_RETIRED",
                help_text="Use cash equivalent in local currency",
                max_length=25,
                verbose_name="Do you know about how much is this worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsproperty",
            name="land_owner",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Do not know"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="Do you own any land.",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsproperty",
            name="land_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(999999999),
                ],
                verbose_name="About how much this is worth in total?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsproperty",
            name="land_value_known",
            field=models.CharField(
                default="QUESTION_RETIRED",
                help_text="Use cash equivalent in local currency",
                max_length=25,
                verbose_name="Do you know about how much this is worth in total?",
            ),
        ),
    ]



# Generated by Django 4.2.3 on 2023-08-02 23:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_he", "0002_historicalreligions_historicalnationalities_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="ethnicities",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="historicalethnicities",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="historicalinsurancetypes",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="historicalnationalities",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="historicalreligions",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="insurancetypes",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="nationalities",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
        migrations.AddField(
            model_name="religions",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



ALL_WINDOWS_SCREENED = "1"
BROTHER_SISTER = "BROTHER_SISTER"
CHF = "chf"
FAMILY_OWNED = "family_owned"
GRANDCHILD = "GRANDCHILD"
JOINT_OWNED = "joint_owned"
NHIF = "nhif"
NON_FAMILY_OWNED = "nonfamily_owned"
OWNER = "owner"
PARENT = "PARENT"
PARENTINLAW = "PARENTINLAW"
SOME_WINDOWS_SCREENED = "2"
SON_DAUGHTER = "SON_DAUGHTER"
SON_DAUGHTERINLAW = "SON_DAUGHTERINLAW"
WIFE_HUSBAND = "WIFE_HUSBAND"



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import YES_NO_DONT_KNOW_DWTA
from edc_constants.constants import NOT_APPLICABLE, QUESTION_RETIRED

from edc_he.choices import LAND_AREA_UNITS


class PropertyModelMixin(models.Model):
    land_owner = models.CharField(
        verbose_name="Do you own any land.",
        max_length=25,
        choices=YES_NO_DONT_KNOW_DWTA,
    )

    # QUESTION_RETIRED
    land_value_known = models.CharField(
        verbose_name="Do you know about how much is this worth in total?",
        max_length=25,
        default=QUESTION_RETIRED,
        help_text="Use cash equivalent in local currency",
    )

    land_value = models.IntegerField(
        verbose_name="About how much is this worth in total?",
        validators=[MinValueValidator(1), MaxValueValidator(999999999)],
        null=True,
        blank=True,
        help_text="Use cash equivalent in local currency",
    )

    land_surface_area = models.IntegerField(
        verbose_name="Surface area",
        validators=[MinValueValidator(1), MaxValueValidator(999999999)],
        null=True,
        blank=True,
    )

    land_surface_area_units = models.CharField(
        verbose_name="Surface area (units)",
        max_length=15,
        choices=LAND_AREA_UNITS,
        default=NOT_APPLICABLE,
        null=True,
        blank=True,
    )

    land_additional = models.CharField(
        verbose_name="Do you own any other property other than your primary dwelling?",
        max_length=25,
        choices=YES_NO_DONT_KNOW_DWTA,
    )

    # QUESTION_RETIRED
    land_additional_known = models.CharField(
        verbose_name="Do you know about how much is this worth in total?",
        max_length=25,
        default=QUESTION_RETIRED,
        help_text="Use cash equivalent in local currency",
    )

    land_additional_value = models.IntegerField(
        verbose_name="About how much is this worth in total?",
        validators=[MinValueValidator(1), MaxValueValidator(999999999)],
        null=True,
        blank=True,
        help_text="Use cash equivalent in local currency",
    )

    class Meta:
        abstract = True



from edc_constants.constants import (
    DONT_KNOW,
    DWTA,
    MONTHLY,
    NOT_APPLICABLE,
    OPTION_RETIRED,
    OTHER,
    PRIMARY,
    SECONDARY,
    TERTIARY,
    WEEKLY,
    YEARLY,
)

from .constants import (
    ALL_WINDOWS_SCREENED,
    BROTHER_SISTER,
    FAMILY_OWNED,
    GRANDCHILD,
    JOINT_OWNED,
    NON_FAMILY_OWNED,
    OWNER,
    PARENT,
    PARENTINLAW,
    SOME_WINDOWS_SCREENED,
    SON_DAUGHTER,
    SON_DAUGHTERINLAW,
    WIFE_HUSBAND,
)

EDUCATION_CERTIFICATES_CHOICES = (
    (PRIMARY, "Primary Certificate"),
    (SECONDARY, "Secondary Certificate"),
    (TERTIARY, "post-Secondary/Tertiary/College"),
    (OTHER, "Other, specify ..."),
    (NOT_APPLICABLE, "Not applicable, never went to school"),
)

RELATIONSHIP_CHOICES = (
    (WIFE_HUSBAND, "Wife/Husband"),
    (SON_DAUGHTER, "Son/Daughter"),
    (SON_DAUGHTERINLAW, "Son/Daughter-in-law"),
    (GRANDCHILD, "Grandchild"),
    (PARENT, "Parent"),
    (PARENTINLAW, "Parent-in-law"),
    (BROTHER_SISTER, "Brother/Sister"),
    (OTHER, "Other, specify ..."),
    (DONT_KNOW, "Don’t know"),
    (NOT_APPLICABLE, "Not applicable"),
)

EMPLOYMENT_STATUS_CHOICES = (
    ("1", "Full time employed"),
    ("2", "Regular part time employed "),
    ("3", "Irregular/ occasional/ day worker employment"),
    ("4", "Non-paid/ voluntary role "),
    ("5", "Student"),
    ("6", "Homemaker"),
    ("7", "Unemployed (able to work)"),
    ("8", "Unemployed (unable to work)"),
    (DONT_KNOW, "Don’t know"),
    (NOT_APPLICABLE, "Not applcable"),
)


MARITAL_CHOICES = (
    ("1", "Never Married (but not co-habiting)"),
    ("2", "Co-habiting"),
    ("3", "Currently Married"),
    ("4", "Separated/Divorced"),
    ("5", "Widowed"),
    (OTHER, "Other, specify ..."),
    (DONT_KNOW, "Don’t know"),
    (NOT_APPLICABLE, "Not applicable"),
)


RESIDENCE_OWNERSHIP_CHOICES = (
    ("renter", "Rent"),
    (OWNER, "Own themselves"),
    (FAMILY_OWNED, "Owned by someone else in family"),
    (NON_FAMILY_OWNED, "Owned by someone else other than family member"),
    (JOINT_OWNED, "Owned together with someone"),
)

WATER_SOURCE_CHOICES = (
    ("piped_into_plot", "Piped into dwelling/yard plot"),
    ("piped_to_neighbour", "Piped to neighbour"),
    ("standpipe", "Public tap/standpipe"),
    ("borehole", "Tube well or borehole"),
    ("protected_well", "Protected dug well"),
    ("protected_spring", "Protected spring"),
    ("rainwater", "Rainwater"),
    (
        "bottled_water_improved",
        "Bottled water, improved source for cooking/hand washing (1-7)",
    ),
    ("unprotected_well", "Unprotected dug well"),
    ("unprotected_spring", "Unprotected spring"),
    ("tanker", "Tanker truck/cart with small tank"),
    ("surface_water", "Surface water (river etc.)"),
    (
        "bottled_water_unimproved",
        "Bottle water, unimproved source for cooking/hand washing (9-12)",
    ),
    (OTHER, "Other, specify ..."),
)


WATER_OBTAIN_CHOICES = (
    ("on_premises", "Water on premises (includes water piped to a neighbour)"),
    ("less_30min", "Less than 30 minutes"),
    ("greater_30min", "30 minutes or longer"),
    (DONT_KNOW, "Don’t know"),
)


TOILET_CHOICES = (
    ("1", "1. Flush/pour flush to piped sewer system – private"),
    ("2", "2. Flush/pour flush to septic tank – private "),
    ("3", "3. Flush/pour flush to pit latrine – private"),
    ("4", "4. Ventilated improved pit (VIP) latrine – private "),
    ("5", "5. Pit latrine with slab – private"),
    ("6", "6. Composting toilet – private"),
    ("7", "7. EcoSan – private"),
    ("8", "8. Flush/pour flush to piped sewer system – shared"),
    ("9", "9. Flush/pour flush to septic tank – shared"),
    ("10", "10. Flush/pour flush to pit latrine – shared"),
    ("11", "11. Ventilated improved pit (VIP) latrine – shared"),
    ("12", "12. Pit latrine with slab – shared"),
    ("13", "13. Composting toilet – shared"),
    ("14", "14. EcoSan – shared"),
    ("15", "15. Flush/pour flush not to sewer/septic tank/pit latrine"),
    ("16", "16. Pit latrine with slab (non-washable)"),
    ("17", "17. Pit latrine without slab/open pit"),
    ("18", "18. Bucket"),
    ("19", "19. Hanging toilet/hanging latrine"),
    ("20", "20. Open defecation (no facility/bush/field)"),
    (OTHER, "Other, specify ..."),
)

ROOF_MATERIAL_CHOICES = (
    ("1", "Thatch, Straw"),
    ("2", "Mud and poles"),
    ("3", "Tin"),
    ("4", "Wood"),
    ("5", "Iron sheet"),
    ("6", "Tiles "),
    ("7", "Cement"),
    (OTHER, "Other, specify ..."),
)

EAVES_CHOICES = (
    ("1", "All eaves closed"),
    ("2", "All eaves open"),
    ("3", "Partially closed"),
)

EXTERNAL_WALL_MATERIALS_CHOICES = (
    ("1", "Thatch, Straw"),
    ("2", "Mud and poles"),
    ("3", "Timber"),
    (OPTION_RETIRED, "Un-burnt bricks"),
    ("5", "Bricks with mud"),
    ("6", "Bricks with cement"),
    ("7", "Cement blocks"),
    ("8", "Stone"),
    (OTHER, "Other, specify ..."),
)

WINDOW_MATERIAL_CHOICES = (
    ("1", "Glass"),
    ("2", "Bags"),
    ("3", "Wood"),
    ("4", "Iron/metal"),
    ("5", "Screens"),
    ("6", "No windows"),
    (OTHER, "Other, specify ..."),
)

WINDOW_SCREENING_CHOICES = (
    (ALL_WINDOWS_SCREENED, "All windows screened"),
    ("2", "No windows screened"),
    (SOME_WINDOWS_SCREENED, "Some windows screened"),
)

WINDOW_SCREENING_TYPE_CHOICES = (
    ("1", "Wire mesh"),
    ("2", "Old bednet"),
    ("3", "No windows screened"),
    ("4", "No windows"),
    (NOT_APPLICABLE, "Not applicable"),
)

FLOOR_MATERIALS_CHOICES = (
    ("6", "Earth, sand"),
    ("7", "Dung, wood, planks, palm, bamboo"),
    ("8", "Parquet, polished wood, vinyl, asphalt strips"),
    ("9", "Ceramic tiles"),
    ("10", "Cement"),
    ("11", "Carpet"),
    (OTHER, "Other, specify ..."),
)

LAND_AREA_UNITS = (
    ("hectares", "hectares"),
    ("acres", "acres"),
    ("sq_meters", "sq. meters"),
    (NOT_APPLICABLE, "Not applicable"),
)
LIGHTING_CHOICES = (
    ("1", "Electricity"),
    ("2", "Paraffin, kerosene or gas lantern "),
    ("3", "Firewood"),
    ("4", "Candle"),
    (OTHER, "Other, specify ..."),
)

COOKING_FUEL_CHOICES = (
    ("1", "Electricity"),
    ("2", "LPG/natural gas/biogas"),
    ("3", "Kerosene"),
    ("4", "Charcoal"),
    ("5", "Wood"),
    ("6", "Coal/lignite, straw/shrubs/grass. agricultural crop, animal dung"),
    ("7", "No food cooked in the household"),
    (OTHER, "Other, specify ..."),
)

INCOME_TIME_ESTIMATE_CHOICES = (
    (WEEKLY, "as weekly income"),
    (MONTHLY, "as monthly income"),
    (YEARLY, "as yearly income"),
    (DONT_KNOW, "Don’t know"),
    (DWTA, "Don’t want to answer"),
    (NOT_APPLICABLE, "Not applicable"),
)

STATUS = (
    ("1", "Very good"),
    ("2", "Good"),
    ("3", "Moderate"),
    ("4", "Bad"),
    ("5", "Very bad"),
    (DWTA, "Don’t want to answer"),
)

FINANCIAL_STATUS = (
    ("1", "Among most wealthy"),
    ("2", "Above average "),
    ("3", "Average wealth"),
    ("4", "Below average"),
    ("5", "Among least wealthy"),
    (DWTA, "Don’t want to answer"),
)

REMITTANCE_CURRENCY_CHOICES = (
    ("USD", "USD"),
    ("GBP", "GBP"),
    (OTHER, "Other, specify ..."),
    (DONT_KNOW, "Dont' know"),
)



from edc_constants.constants import DONT_KNOW, NONE, OTHER
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidator


class HealthEconomicsPatientFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        self.validate_other_specify(field="pat_religion")
        self.validate_other_specify(field="pat_ethnicity")
        self.validate_other_specify(field="pat_education")
        self.validate_other_specify(field="pat_marital_status")
        self.m2m_single_selection_if(DONT_KNOW, NONE, m2m_field="pat_insurance")
        self.m2m_other_specify(
            OTHER, m2m_field="pat_insurance", field_other="pat_insurance_other"
        )



from .assets import HealthEconomicsAssets
from .household_head import HealthEconomicsHouseholdHead
from .income import HealthEconomicsIncome
from .list_models import (
    Education,
    EmploymentType,
    Ethnicities,
    InsuranceTypes,
    Nationalities,
    Religions,
)
from .patient import HealthEconomicsPatient
from .property import HealthEconomicsProperty



from django.conf import settings
from django.db import models
from edc_crf.model_mixins import CrfModelMixin, SingletonCrfModelMixin
from edc_model.models import BaseUuidModel

from ..model_mixins import PatientModelMixin


class HealthEconomicsPatient(
    SingletonCrfModelMixin,
    PatientModelMixin,
    CrfModelMixin,
    BaseUuidModel,
):
    subject_visit = models.OneToOneField(
        settings.SUBJECT_VISIT_MODEL,
        on_delete=models.PROTECT,
        related_name="he_patient_subjectvisit",
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Health Economics: Patient"
        verbose_name_plural = "Health Economics: Patient"



from django.test import TestCase
from edc_constants.constants import NOT_APPLICABLE, SECONDARY
from edc_form_validators import FormValidatorTestCaseMixin
from edc_form_validators.form_validator import FormValidator
from edc_utils import get_utcnow

from ...form_validators import SimpleFormValidatorMixin
from ..forms import HealthEconomicsForm as BaseHealthEconomicsForm
from ..models import HealthEconomics


class HealthEconomicsFormValidator(SimpleFormValidatorMixin, FormValidator):
    def clean(self) -> None:
        self.clean_education()

    @property
    def age_in_years(self) -> int:
        return 25


class HealthEconomicsForm(BaseHealthEconomicsForm):
    form_validator_cls = HealthEconomicsFormValidator

    class Meta:
        model = HealthEconomics
        fields = "__all__"


class TestHe(FormValidatorTestCaseMixin, TestCase):
    form_validator_cls = HealthEconomicsFormValidator

    def test_form_validator_education(self):
        cleaned_data = {
            "report_datetime": get_utcnow(),
            "education_in_years": 0,
            "education_certificate": NOT_APPLICABLE,
            "primary_school": NOT_APPLICABLE,
            "secondary_school": NOT_APPLICABLE,
            "higher_education": NOT_APPLICABLE,
        }
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertEqual({}, form._errors)

        cleaned_data.update({"education_in_years": None, "education_certificate": None})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertIn("education_in_years", form._errors)
        self.assertIn("education_certificate", form._errors)

        cleaned_data.update({"education_in_years": 0, "education_certificate": None})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertNotIn("education_in_years", form._errors)
        self.assertIn("education_certificate", form._errors)

        cleaned_data.update({"education_in_years": 0, "education_certificate": SECONDARY})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertIn("education_certificate", form._errors)

        cleaned_data.update({"education_in_years": 0, "education_certificate": "blah"})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertIn("education_certificate", form._errors)

        cleaned_data.update({"education_in_years": 0, "education_certificate": NOT_APPLICABLE})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertNotIn("education_certificate", form._errors)

        cleaned_data.update({"education_in_years": 1, "education_certificate": None})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertIn("education_certificate", form._errors)

        cleaned_data.update({"education_in_years": 1, "education_certificate": "blah"})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertIn("education_certificate", form._errors)

        cleaned_data.update({"education_in_years": 7, "education_certificate": SECONDARY})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertNotIn("education_certificate", form._errors)

        # years of education exceeds age
        cleaned_data.update({"education_in_years": 100, "education_certificate": None})
        form = HealthEconomicsForm(data=cleaned_data)
        form.is_valid()
        self.assertIn("education_in_years", form._errors)



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import YES_NO_NA
from edc_model.models import OtherCharField

from ...choices import EDUCATION_CERTIFICATES_CHOICES


class HealthEconomicsEducationModelMixin(models.Model):
    education_in_years = models.IntegerField(
        verbose_name="How many years of education did you complete?",
        validators=[MinValueValidator(0), MaxValueValidator(30)],
    )

    education_certificate = models.CharField(
        verbose_name="What is your highest education certificate?",
        max_length=50,
        choices=EDUCATION_CERTIFICATES_CHOICES,
        null=True,
        blank=False,
    )

    education_certificate_other = OtherCharField()

    education_certificate_tertiary = models.CharField(
        verbose_name=(
            "If your highest education certificate above is "
            "`Tertiary`, what type of tertiary certificate?"
        ),
        max_length=50,
        null=True,
        blank=True,
    )

    primary_school = models.CharField(
        verbose_name="Did you go to primary/elementary school?",
        max_length=15,
        choices=YES_NO_NA,
    )

    primary_school_in_years = models.IntegerField(
        verbose_name="If YES, for how many years",
        validators=[MinValueValidator(0), MaxValueValidator(10)],
        null=True,
        blank=True,
    )

    secondary_school = models.CharField(
        verbose_name="Did you go to secondary school?", max_length=15, choices=YES_NO_NA
    )

    secondary_school_in_years = models.IntegerField(
        verbose_name="If YES, for how many years",
        validators=[MinValueValidator(0), MaxValueValidator(10)],
        null=True,
        blank=True,
    )

    higher_education = models.CharField(
        verbose_name="Did you go to higher education?", max_length=15, choices=YES_NO_NA
    )

    higher_education_in_years = models.IntegerField(
        verbose_name="If YES, for how many years",
        validators=[MinValueValidator(0), MaxValueValidator(20)],
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True



from django.contrib import admin

education_fieldset = (
    "Education",
    {
        "fields": (
            "education_in_years",
            "education_certificate",
            "education_certificate_other",
            "education_certificate_tertiary",
            "primary_school",
            "primary_school_in_years",
            "secondary_school",
            "secondary_school_in_years",
            "higher_education",
            "higher_education_in_years",
        )
    },
)

education_radio_fields = {
    "higher_education": admin.VERTICAL,
    "marital_status": admin.VERTICAL,
    "primary_school": admin.VERTICAL,
    "secondary_school": admin.VERTICAL,
    "education_certificate": admin.VERTICAL,
}



from django.contrib import admin
from edc_list_data.admin import ListModelAdminMixin

from .admin_site import edc_he_admin
from .models import Ethnicities, InsuranceTypes, Nationalities, Religions


@admin.register(InsuranceTypes, site=edc_he_admin)
class InsuranceTypesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Nationalities, site=edc_he_admin)
class NationalitiesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Ethnicities, site=edc_he_admin)
class EthnicitiesAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass


@admin.register(Religions, site=edc_he_admin)
class ReligionsAdmin(ListModelAdminMixin, admin.ModelAdmin):
    pass



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin, CrfSingletonModelFormMixin

from ..form_validators.assets_form_validator import HealthEconomicsAssetsFormValidator
from ..models import HealthEconomicsAssets
from .modelform_mixins import HealthEconomicsModelFormMixin


class HealthEconomicsAssetsForm(
    CrfSingletonModelFormMixin,
    HealthEconomicsModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = HealthEconomicsAssetsFormValidator

    def clean(self):
        self.raise_if_singleton_exists()
        return super().clean()

    class Meta:
        model = HealthEconomicsAssets
        fields = "__all__"



from .assets_model_mixin import AssetsModelMixin
from .factory import assets_model_mixin_factory, income_model_mixin_factory
from .household_head_model_mixin import HouseholdHeadModelMixin
from .household_model_mixin import HouseholdModelMixin
from .income_model_mixin import IncomeModelMixin
from .patient_model_mixin import PatientModelMixin
from .property_model_mixin import PropertyModelMixin
from .simple import HealthEconomicsEducationModelMixin



from edc_constants.constants import YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidator

from ..constants import (
    ALL_WINDOWS_SCREENED,
    FAMILY_OWNED,
    JOINT_OWNED,
    NON_FAMILY_OWNED,
    OWNER,
    SOME_WINDOWS_SCREENED,
)


class HealthEconomicsAssetsFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        self.applicable_if(
            OWNER,
            FAMILY_OWNED,
            NON_FAMILY_OWNED,
            JOINT_OWNED,
            field="residence_ownership",
            field_applicable="dwelling_value_known",
        )
        self.required_if(YES, field="dwelling_value_known", field_required="dwelling_value")
        self.validate_other_specify(field="water_source")
        self.validate_other_specify(field="toilet")
        self.validate_other_specify(field="roof_material")
        self.validate_other_specify(field="external_wall_material")
        self.validate_other_specify(field="external_window_material")
        self.applicable_if(
            ALL_WINDOWS_SCREENED,
            SOME_WINDOWS_SCREENED,
            field="window_screens",
            field_applicable="window_screen_type",
        )
        self.validate_other_specify(field="floor_material")
        self.validate_other_specify(field="light_source")
        self.validate_other_specify(field="cooking_fuel")



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.db.models import PROTECT
from edc_constants.choices import GENDER, YES_NO
from edc_constants.constants import NOT_APPLICABLE
from edc_model_fields.fields import OtherCharField

from ..choices import EMPLOYMENT_STATUS_CHOICES, MARITAL_CHOICES, RELATIONSHIP_CHOICES


class HouseholdHeadModelMixin(models.Model):
    hoh = models.CharField(
        verbose_name="Are you the household head?",
        max_length=15,
        choices=YES_NO,
    )

    hoh_gender = models.CharField(
        verbose_name="Is the household head female or male?",
        max_length=15,
        choices=GENDER,
        help_text="Note: if you are the household head, indicate your gender",
    )

    hoh_age = models.IntegerField(
        verbose_name="How old is the household head?",
        validators=[MinValueValidator(18), MaxValueValidator(110)],
        help_text=(
            "In years. Note: if you are the household head, indicate your age as of today"
        ),
    )

    relationship_to_hoh = models.CharField(
        verbose_name="What is your relationship to the household head?",
        max_length=25,
        choices=RELATIONSHIP_CHOICES,
        default=NOT_APPLICABLE,
        help_text="Not applicable if patient is head of household",
    )

    relationship_to_hoh_other = OtherCharField(
        verbose_name="If OTHER relationship, specify ...",
    )

    hoh_religion = models.ForeignKey(
        "edc_he.Religions",
        verbose_name="How would you describe the household head’s religious orientation?",
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text="Not applicable if patient is head of household",
    )

    hoh_religion_other = OtherCharField(
        verbose_name="If OTHER religious orientation, specify ...",
    )

    hoh_ethnicity = models.ForeignKey(
        "edc_he.ethnicities",
        verbose_name="What is the household head’s ethnic background?",
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text="Not applicable if patient is head of household",
    )

    hoh_ethnicity_other = OtherCharField(
        verbose_name="If OTHER ethnic background, specify ...",
    )

    hoh_education = models.ForeignKey(
        "edc_he.education",
        verbose_name="Highest level of education completed by the household head?",
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text="Not applicable if patient is head of household",
    )

    hoh_education_other = OtherCharField(
        verbose_name="If OTHER education, specify ...",
    )

    hoh_employment_status = models.CharField(
        verbose_name="Household head’s employment status",
        max_length=25,
        choices=EMPLOYMENT_STATUS_CHOICES,
        default=NOT_APPLICABLE,
        help_text="Not applicable if patient is head of household",
    )

    hoh_employment_type = models.ForeignKey(
        "edc_he.employmenttype",
        verbose_name="Household head’s type of employment",
        related_name="+",
        on_delete=PROTECT,
        null=True,
        blank=False,
        help_text="Not applicable if patient is head of household",
    )

    hoh_employment_type_other = OtherCharField(
        verbose_name="If OTHER type of employment, specify ...",
        max_length=100,
    )

    hoh_marital_status = models.CharField(
        verbose_name="Household head’s marital status",
        max_length=25,
        choices=MARITAL_CHOICES,
        default=NOT_APPLICABLE,
        help_text="Not applicable if patient is head of household",
    )

    hoh_marital_status_other = OtherCharField(
        verbose_name="If OTHER marital status, specify ...",
    )

    hoh_insurance = models.ManyToManyField(
        "edc_he.insurancetypes",
        verbose_name="Household head’s health insurance and ‘club’ status ",
        related_name="+",
        help_text="Not applicable if patient is head of household",
    )

    hoh_insurance_other = OtherCharField(
        verbose_name="If OTHER, specify ...",
    )

    # not used
    hoh_religion_old = models.CharField(
        verbose_name="How would you describe the household head’s religious orientation?",
        max_length=25,
    )

    # not used
    hoh_employment_type_old = models.CharField(
        verbose_name="Household head’s type of employment",
        max_length=25,
    )

    # not used
    hoh_education_old = models.CharField(
        verbose_name="Highest level of education completed by the household head?",
        max_length=25,
    )

    # not used
    hoh_ethnicity_old = models.CharField(
        verbose_name="What is the household head’s ethnic background?",
        max_length=25,
    )

    class Meta:
        abstract = True



from edc_constants.constants import YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidator


class HealthEconomicsPropertyFormValidator(
    CrfFormValidatorMixin,
    FormValidator,
):
    def clean(self):
        self.required_if(YES, field="land_owner", field_required="land_value")
        self.required_if(YES, field="land_owner", field_required="land_surface_area")
        self.applicable_if_true(
            self.cleaned_data.get("land_surface_area"),
            field_applicable="land_surface_area_units",
        )
        self.required_if(YES, field="land_additional", field_required="land_additional_value")



from __future__ import annotations

from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_constants.choices import YES_NO_DONT_KNOW_DWTA
from edc_constants.constants import NOT_APPLICABLE

from ...choices import INCOME_TIME_ESTIMATE_CHOICES

default_field_data = {
    "wages": "Income from wages, salary from job",
    "selling": "Earnings from selling, trading or hawking products?",
    "rental_income": "Income from rental of property?",
    "pension": (
        (
            "State old-age (veteran's/civil service) pension*, contributory pension "
            "fund, provident fund or social security benefit?"
        ),
        "Pensions by work",
    ),
    "ngo_assistance": "Assistance from nongovernmental organization",
    "interest": (
        "Interest, dividends",
        "(for example, from savings account or fixed deposits)?",
    ),
    "internal_remit": (
        "Money transfers from family members or friends residing inside the country"
    ),
    "external_remit": (
        "Money transfers from family members or friends residing outside the country"
    ),
    "more_sources": "Do you have additional sources of income not included above?",
}


def income_model_mixin_factory(field_data: dict[str, str] | None = None):
    field_data = field_data or default_field_data

    class AbstractModel(models.Model):
        class Meta:
            abstract = True

    opts = {}
    for field_name, prompt in field_data.items():
        try:
            prompt, help_text = prompt
        except ValueError:
            help_text = None
        opts.update(
            {
                field_name: models.CharField(
                    verbose_name=prompt,
                    max_length=15,
                    choices=YES_NO_DONT_KNOW_DWTA,
                    help_text=help_text,
                ),
                f"{field_name}_value_known": models.CharField(
                    verbose_name="Over which <u>time period</u> are you able to estimate?",
                    max_length=15,
                    choices=INCOME_TIME_ESTIMATE_CHOICES,
                    default=NOT_APPLICABLE,
                ),
                f"{field_name}_value": models.IntegerField(
                    verbose_name=(
                        "Estimated <u>total amount of income</u> from this source over the "
                        "time period from above"
                    ),
                    validators=[MinValueValidator(1), MaxValueValidator(999999999)],
                    null=True,
                    blank=True,
                    help_text="Use cash equivalent in local currency",
                ),
            }
        )
    for fld_name, fld_cls in opts.items():
        AbstractModel.add_to_class(fld_name, fld_cls)

    return AbstractModel



from django import forms
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidatorMixin
from edc_form_validators.form_validator import FormValidator

from edc_he.form_validators import SimpleFormValidatorMixin

from .models import HealthEconomics


class HealthEconomicsFormValidator(
    CrfFormValidatorMixin, SimpleFormValidatorMixin, FormValidator
):
    def clean(self) -> None:
        self.clean_education()


class HealthEconomicsForm(FormValidatorMixin, forms.ModelForm):
    form_validator_cls = HealthEconomicsFormValidator

    class Meta:
        model = HealthEconomics
        fields = "__all__"



from edc_model.models import BaseUuidModel

from edc_he.model_mixins import HealthEconomicsEducationModelMixin


class HealthEconomics(
    HealthEconomicsEducationModelMixin,
    BaseUuidModel,
):
    class Meta:
        pass



from __future__ import annotations

from django.db import models
from edc_constants.choices import YES_NO
from edc_constants.constants import NO

default_field_data = {
    "solar_panels": "Solar panels",
    "radio": "Radio",
    "television": "Television",
    "mobile_phone": "Mobile phone",
    "computer": "Computer",
    "telephone": "Non-mobile telephone",
    "fridge": "Fridge",
    "generator": "Generator",
    "iron": "Flat iron",
    "bicycle": "Bicycle",
    "motorcycle": "Motorcycle/scooter (PikiPiki/Boda Boda)",
    "dala_dala": "Dala Dala",
    "car": "Car",
    "motorboat": "Boat with a motor",
    "large_livestock": "Large Livestock (e.g. cows, bulls, other cattle, horses, donkeys)",
    "small_animals": "Small animals (goats, sheep, chickens or other poultry, etc)",
    "shop": "A business or shop",
}


def assets_model_mixin_factory(field_data: dict[str, str] | None = None):
    field_data = field_data or default_field_data

    class AbstractModel(models.Model):
        class Meta:
            abstract = True

    opts = {}
    for field_name, prompt in field_data.items():
        opts.update(
            {
                field_name: models.CharField(
                    verbose_name=prompt,
                    max_length=15,
                    choices=YES_NO,
                    default=NO,
                ),
            }
        )
    for fld_name, fld_cls in opts.items():
        AbstractModel.add_to_class(fld_name, fld_cls)

    return AbstractModel



from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple


def get_income_fieldsets_tuple() -> list[tuple]:
    fieldsets = []
    sources = [
        ("wages", "Wages / Salary"),
        ("selling", "Selling, trading or hawking"),
        ("rental_income", "Rental income"),
        ("pension", "Pension, etc."),
        ("ngo_assistance", "NGO assistance"),
        ("interest", "Interest and dividends"),
        ("internal_remit", "Money transfers (domestic)"),
        ("external_remit", "Remittances (from outside the country)"),
        ("more_sources", "Other sources of income"),
    ]
    for fld, label in sources:
        fields = [
            fld,
            f"{fld}_value_known",
            f"{fld}_value",
        ]
        if fld == "external_remit":
            fields.extend(["external_remit_currency", "external_remit_currency_other"])
        if fld == "more_sources":
            fields.insert(1, "more_sources_other")

        fieldsets.append(
            (
                label,
                {
                    "description": format_html(
                        "Estimate the total amount of income from this source "
                        "for the <b>household</b> over the time period indicated"
                    ),
                    "fields": tuple(fields),
                },
            )
        )
    return fieldsets


class HealthEconomicsIncomeModelAdminMixin:
    form = None

    additional_instructions = format_html(
        "<H5><B><font color='orange'>Interviewer to read</font></B></H5>"
        "<P>We want to learn about the household and we use these questions "
        "to get an understanding of wealth and opportunities in the community.</P>"
        "<p>Now, I will ask about <b>income for the household</b> from paid work or "
        "other sources.</p><p>I know it may be difficult to calculate those figures, "
        "but please do try to give amounts as accurately as possible. Remember "
        "that <b>all information will be kept strictly confidential</b>. This "
        "information is important to assess overall health and well-being of "
        "people in your household, compared to other similar households.</p><p>"
        "I am now going to read you a list of possible sources of income. "
        "Thinking over the last 12 months, can you tell me what the average "
        "earnings of the household have been per week or per month or per year? "
        "Please tell me whichever time period that is easier for you.</p>"
    )
    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        *get_income_fieldsets_tuple(),
        (
            "Dependents, financial status",
            {
                "fields": (
                    "external_dependents",
                    "income_enough",
                    "financial_status",
                    "financial_status_compare",
                )
            },
        ),
        (
            "Debt / Loans",
            {
                "description": format_html(
                    "<H5><B><font color='orange'>Interviewer to read</font></B></H5>"
                    "<p>Now, I will ask about the total current debt/loans for the household. "
                    "I know it may be difficult to calculate this, but please do try to "
                    "give amounts as accurately as possible. Remember that all information "
                    "will be kept strictly confidential. <B>It is not possible to offer "
                    "any help for your debts through this project</B>, but this information "
                    "will contribute to research and is important to assess the overall "
                    "health and well-being of people in your household, compared to other "
                    "similar households.</p>"
                ),
                "fields": (
                    "household_debt",
                    "household_debt_value",
                ),
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "wages": admin.VERTICAL,
        "wages_value_known": admin.VERTICAL,
        "selling": admin.VERTICAL,
        "selling_value_known": admin.VERTICAL,
        "rental_income": admin.VERTICAL,
        "rental_income_value_known": admin.VERTICAL,
        "pension": admin.VERTICAL,
        "pension_value_known": admin.VERTICAL,
        "ngo_assistance": admin.VERTICAL,
        "ngo_assistance_value_known": admin.VERTICAL,
        "interest": admin.VERTICAL,
        "interest_value_known": admin.VERTICAL,
        "internal_remit": admin.VERTICAL,
        "internal_remit_value_known": admin.VERTICAL,
        "external_remit": admin.VERTICAL,
        "external_remit_value_known": admin.VERTICAL,
        "external_remit_currency": admin.VERTICAL,
        "more_sources": admin.VERTICAL,
        "more_sources_value_known": admin.VERTICAL,
        "income_enough": admin.VERTICAL,
        "financial_status": admin.VERTICAL,
        "financial_status_compare": admin.VERTICAL,
        "household_debt": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }



# Generated by Django 4.2.3 on 2023-08-07 23:19

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_he", "0013_alter_healtheconomicsassets_bedrooms_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="healtheconomicsincome",
            old_name="external_remittance",
            new_name="external_remit",
        ),
        migrations.RenameField(
            model_name="healtheconomicsincome",
            old_name="external_remittance_value_known",
            new_name="external_remit_value_known",
        ),
        migrations.RenameField(
            model_name="healtheconomicsincome",
            old_name="internal_remittance",
            new_name="internal_remit",
        ),
        migrations.RenameField(
            model_name="healtheconomicsincome",
            old_name="internal_remittance_value_known",
            new_name="internal_remit_value_known",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicsincome",
            old_name="external_remittance",
            new_name="external_remit",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicsincome",
            old_name="external_remittance_value_known",
            new_name="external_remit_value_known",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicsincome",
            old_name="internal_remittance",
            new_name="internal_remit",
        ),
        migrations.RenameField(
            model_name="historicalhealtheconomicsincome",
            old_name="internal_remittance_value_known",
            new_name="internal_remit_value_known",
        ),
        migrations.RemoveField(
            model_name="healtheconomicsincome",
            name="external_remittance_value",
        ),
        migrations.RemoveField(
            model_name="healtheconomicsincome",
            name="internal_remittance_value",
        ),
        migrations.RemoveField(
            model_name="historicalhealtheconomicsincome",
            name="external_remittance_value",
        ),
        migrations.RemoveField(
            model_name="historicalhealtheconomicsincome",
            name="internal_remittance_value",
        ),
        migrations.AddField(
            model_name="healtheconomicsincome",
            name="external_remit_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AddField(
            model_name="healtheconomicsincome",
            name="internal_remit_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsincome",
            name="external_remit_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AddField(
            model_name="historicalhealtheconomicsincome",
            name="internal_remit_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="dwelling_value_known",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Do not know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="in local currency",
                max_length=25,
                verbose_name="If the owner [you] were to sell this dwelling today, do you know the approximate value?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsassets",
            name="rooms",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many rooms does your dwelling have in total, without counting the bathrooms/ toilets or hallways/passageways?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="interest_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="more_sources_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="ngo_assistance_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="pension_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="rental_income_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="selling_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="wages_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="dwelling_value_known",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("dont_know", "Do not know"),
                    ("DWTA", "Don't want to answer"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="in local currency",
                max_length=25,
                verbose_name="If the owner [you] were to sell this dwelling today, do you know the approximate value?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsassets",
            name="rooms",
            field=models.IntegerField(
                validators=[
                    django.core.validators.MinValueValidator(1),
                    django.core.validators.MaxValueValidator(30),
                ],
                verbose_name="How many rooms does your dwelling have in total, without counting the bathrooms/ toilets or hallways/passageways?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="interest_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="more_sources_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="ngo_assistance_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="pension_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="rental_income_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="selling_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="wages_value",
            field=models.IntegerField(
                blank=True,
                help_text="Use cash equivalent in local currency",
                null=True,
                verbose_name="Estimated <u>total amount of income</u> from this source over the time period from above",
            ),
        ),
    ]
